;; Generated package description from url-scgi.el  -*- no-byte-compile: t -*-
(define-package "url-scgi" "0.9.0.20230130.74744" "SCGI support for url.el" '((emacs "24.3")) :commit "ccf30c3647cd6a560cadc795bd92769c0a0c9de9" :authors '(("Stefan Kangas" . "stefankangas@gmail.com")) :maintainer '("Stefan Kangas" . "stefankangas@gmail.com") :keywords '("comm" "data" "processes" "scgi") :url "https://github.com/skangas/url-scgi/")
